@@ -7,17 +7,18 @@ from PySide2.QtWidgets import QLabel, QPushButton, QSlider, QStyle, \
     QHBoxLayout, QVBoxLayout, QSizePolicy, QAction, QGridLayout, QDialog, \
     QTabWidget, QSplitter, QScrollArea, QMessageBox
 from PySide2.QtGui import Qt
-from PySide2.QtCore import QObject, QEvent
+from PySide2.QtCore import QObject, QEvent, Signal
 
 import timecode
 import os
+import sys
 from functools import partial
 
 from peyecoder.video_reader import BufferedVideoReader
 from peyecoder.audio_player import VideoAudioPlayer
 from peyecoder.panels import Prescreen, Code, LogTable
-from peyecoder.models import Subject
-from peyecoder.file_utils import load_datafile, save_datafile
+from peyecoder.models import Subject, Occluders
+from peyecoder.file_utils import load_datafile, save_datafile, intify_keys
 from peyecoder.dialogs import SubjectDialog, TimecodeDialog, OccluderDialog, SettingsDialog, CodeComparisonDialog, \
     ReportDialog, ExportDialog, get_save_filename, ReplaceDialog
 from peyecoder.reliability import reliability_report
@@ -30,6 +31,21 @@ TAB_PRESCREEN = 0
 TAB_CODE = 1
 
 
+class JumpSlider(QSlider):
+    """ This subclass of QSlider supports jumping to a position on the slider by clicking on it (while continuing to
+    allow dragging the slider).  Clicking on the slider results in the firing of a new signal 'clicked'.
+    Inspired by https://stackoverflow.com/questions/11132597/qslider-mouse-direct-jump
+    """
+    clicked = Signal(int)
+
+    def mousePressEvent(self, ev):
+        """ Jump to click position """
+        v = QStyle.sliderValueFromPosition(self.minimum(), self.maximum(), ev.x(), self.width())
+        self.setValue(v)
+        super().mousePressEvent(ev)
+        self.clicked.emit(v)
+
+
 class MainEventFilter(QObject):
     def eventFilter(self, obj, event):
         # if event.type() == QEvent.KeyPress:
@@ -44,7 +60,7 @@ class MainEventFilter(QObject):
 
 class MainWindow(QtWidgets.QMainWindow):
 
-    def __init__(self):
+    def __init__(self, argv):
         super().__init__()
 
         self.setFocusPolicy(Qt.ClickFocus)
@@ -129,6 +145,11 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.reset_state()
 
+        # If a filename has been passed as a command line argument, try to open it
+        if len(argv) > 1:
+            if os.path.isfile(argv[1]):
+                self.open_data_file(argv[1])
+
     def prompt_save(self):
         proceed = True
         if self.subject.dirty:
@@ -149,6 +170,30 @@ class MainWindow(QtWidgets.QMainWindow):
         else:
             event.ignore()
 
+    def load_defaults(self):
+        """Load default.vcx if present in working directory."""
+        if getattr(sys, 'frozen', False):
+            # running in a bundle
+            defaultdir = os.path.dirname(sys.executable)
+        else:
+            defaultdir = os.getcwd()
+
+        filename = os.path.join(defaultdir, 'default.vcx')
+        if os.path.isfile(filename):
+            data = load_datafile(filename)
+            d = data['Subject']
+
+            if 'Occluders' in d:
+                self.subject.occluders = Occluders.from_dictlist(d['Occluders'])
+            if 'Settings' in d:
+                if 'Response Keys' in d['Settings']:
+                    d['Settings']['Response Keys'] = intify_keys(d['Settings']['Response Keys'])
+                self.subject.settings.update(d['Settings'])
+
+            print('Loaded settings from {}'.format(filename))
+        else:
+            print('No settings file detected: {}'.format(filename))
+
     def reset_state(self):
         """Initialize state or reset to initial state"""
 
@@ -184,10 +229,12 @@ class MainWindow(QtWidgets.QMainWindow):
         # reset coding controls
         self.prescreen_tab.trial_box.setValue(1)
         self.code_tab.trial_box.setValue(1)
+        self.tab_widget.setCurrentIndex(TAB_CODE)  # setting the current tab also updates the log table
 
-        self.update_log()
         self.reset_info_panel()
 
+        self.load_defaults()
+
         self.open_subject_dialog()
 
     def build_playback_widgets(self):
@@ -203,9 +250,10 @@ class MainWindow(QtWidgets.QMainWindow):
         self.mute_button.clicked.connect(self.toggle_mute)
         self.mute_button.setFocusPolicy(Qt.NoFocus)
 
-        self.position_slider = QSlider(QtCore.Qt.Horizontal)
+        self.position_slider = JumpSlider(QtCore.Qt.Horizontal)
         self.position_slider.setRange(0, 0)
         self.position_slider.sliderMoved.connect(self.setPosition)
+        self.position_slider.clicked.connect(self.setPosition)
         self.position_slider.setFocusPolicy(Qt.NoFocus)
 
         control_layout = QHBoxLayout()
@@ -308,8 +356,6 @@ class MainWindow(QtWidgets.QMainWindow):
         self.tab_widget.currentChanged.connect(self.change_tab)
         self.tab_widget.setFocusPolicy(Qt.NoFocus)
 
-        self.active_tab = TAB_PRESCREEN
-
         return self.tab_widget
 
     def change_tab(self, index):
@@ -324,8 +370,9 @@ class MainWindow(QtWidgets.QMainWindow):
             self.logtable.set_prescreen_labels(self.prescreen_tab.prescreener())
             self.logtable.load_data(self.subject.reasons.render(self.prescreen_tab.prescreener()))
             error_rows, error_trials = self.subject.reasons.error_items(self.prescreen_tab.prescreener())
-            self.logtable.redden_rows(error_rows)
-            if error_trials:
+            # Only display error messages and redden rows if "display both coders" is checked
+            if error_trials and (self.prescreen_tab.prescreener() == 0):
+                self.logtable.redden_rows(error_rows)
                 self.message_box.setText('Mismatch between prescreener 1 and prescreener 2 for trials {}'.format(error_trials))
             else:
                 self.message_box.setText('')
@@ -436,26 +483,32 @@ class MainWindow(QtWidgets.QMainWindow):
         next_step_action = QAction('Skip &forward', self)
         next_step_action.setShortcut(']')
         next_step_action.setShortcutContext(Qt.WidgetShortcut)  # prevent menu item from intercepting this keypress
+        next_step_action.triggered.connect(self.next_step)
 
         prev_step_action = QAction('Skip &backward', self)
         prev_step_action.setShortcut('[')
         prev_step_action.setShortcutContext(Qt.WidgetShortcut)
+        prev_step_action.triggered.connect(self.prev_step)
 
         increment_action = QAction('Increment Trial', self)
         increment_action.setShortcut('+')
         increment_action.setShortcutContext(Qt.WidgetShortcut)
+        increment_action.triggered.connect(partial(self.change_trial, 1))
 
         decrement_action = QAction('Decrement Trial', self)
         decrement_action.setShortcut('-')
         decrement_action.setShortcutContext(Qt.WidgetShortcut)
+        decrement_action.triggered.connect(partial(self.change_trial, -1))
 
         increment_selected_action = QAction('Increment Selected Trials', self)
         increment_selected_action.setShortcut('Alt++')
         increment_selected_action.setShortcutContext(Qt.WidgetShortcut)
+        increment_selected_action.triggered.connect(partial(self.change_selected_trials, 1))
 
         decrement_selected_action = QAction('Decrement Selected Trials', self)
         decrement_selected_action.setShortcut('Alt+-')
         decrement_selected_action.setShortcutContext(Qt.WidgetShortcut)
+        decrement_selected_action.triggered.connect(partial(self.change_selected_trials, -1))
 
         help_url_action = QAction('Online Help', self)
         help_url_action.triggered.connect(self.open_help_url)
@@ -785,14 +838,24 @@ class MainWindow(QtWidgets.QMainWindow):
                 self.message_box.setText(e.strerror)
             self.enable_controls()
 
+            # may need to re-render timestamps of existing events if video framerate is not 30 fps
+            self.update_log()
+
     def open_datafile(self):
         filename, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Open Data File", filter="Data Files (*.vcx)") #, QtCore.QDir.homePath())
+        self.open_data_file(filename)
+
+    def open_data_file(self, filename):
         if filename != '':
             self.reset_state()
             self.subject.from_plist(load_datafile(filename))
             if self.vid:
                 self.subject.events.remove_offset(self.subject.timecode_offsets.get_offset(0))
 
+            # update timecode so stored framerate used to render timecodes (if a video is not loaded)
+            self.timecode = timecode.Timecode(self.subject['Framerate'])
+            self.timecode.drop_frame = False
+
             self.update_log()
             self.update_info_panel()
             if self.subject_dialog:
@@ -929,10 +992,10 @@ class MainWindow(QtWidgets.QMainWindow):
         self.show_frame()  # update (and resize) the display of the current frame
 
 
-def run():
+def run(argv):
     """Run peyecoder application"""
     app = QtWidgets.QApplication([])
-    widget = MainWindow()
+    widget = MainWindow(argv)
     widget.resize(800, 600)
     widget.show()
     return app.exec_()
